package org.heigit.ohsome.oshdb.tool.importer.transform.oshdb;

import java.io.IOException;
import java.util.Collections;
import java.util.List;
import org.heigit.ohsome.oshdb.OSHDBBoundingBox;
import org.heigit.ohsome.oshdb.osm.OSMNode;
import org.heigit.ohsome.oshdb.util.bytearray.ByteArrayOutputWrapper;
import org.heigit.ohsome.oshdb.util.bytearray.ByteArrayWrapper;

public class TransformOSHNode extends OSHNode2 {

  public static TransformOSHNode build(ByteArrayOutputWrapper output, ByteArrayOutputWrapper record,
      ByteArrayOutputWrapper aux, List<OSMNode> versions, final long baseId,
      final long baseTimestamp, final long baseLongitude, final long baseLatitude)
      throws IOException {
    Collections.sort(versions, Collections.reverseOrder());

    output.reset();
    record.reset();

    final long id = versions.get(0).getId();

    OSHNode2.OSHNodeBuilder builder = new OSHNode2.OSHNodeBuilder();

    builder.build(output, aux, versions, baseTimestamp, baseLongitude, baseLatitude,
        Collections.emptyMap(), Collections.emptyMap(), Collections.emptyMap());

    record.writeU64(id - baseId);
    record.writeByteArray(output.array(), 0, output.length());

    return TransformOSHNode.instance(record.array(), 0, record.length(), baseId, baseTimestamp,
        baseLongitude, baseLatitude);
  }

  public static TransformOSHNode instance(final byte[] data, final int offset, final int length,
      final long baseId, final long baseTimestamp, final long baseLongitude,
      final long baseLatitude) throws IOException {

    final ByteArrayWrapper wrapper = ByteArrayWrapper.newInstance(data, offset, length);

    final byte header = 0;

    final long id = wrapper.readU64() + baseId;
    final int dataOffset = wrapper.getPos();
    final int dataLength = length - (dataOffset - offset);

    return new TransformOSHNode(data, offset, length, header, id, baseTimestamp, baseLongitude,
        baseLatitude, dataOffset, dataLength);
  }

  public static TransformOSHNode instance(final byte[] data, final int offset, final int length)
      throws IOException {
    return instance(data, offset, length, 0, 0, 0, 0);
  }

  private TransformOSHNode(final byte[] data, final int offset, final int length, final byte header,
      final long id, final long baseTimestamp, final long baseLongitude, final long baseLatitude,
      final int dataOffset, final int dataLength) {
    super(data, offset, length, header, id, OSHDBBoundingBox.INVALID, baseTimestamp, baseLongitude,
        baseLatitude, new int[0], dataOffset, dataLength);
  }
}
